\# $Id: pthread_cleanup_push.man,v 1.1.1.1 1996/06/29 01:20:50 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_cleanup_push.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:48  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:24  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:05  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_cleanup_push \- Push a cleanup handler.
.br
pthread_cleanup_pop \- Pop, and optionally execute, a cleanup handler.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B void
.br
.BI "pthread_cleanup_push((void *) (*" handler ")(void *" arg ")" ","
.br
.BI "                      void *" arg )
.br
.B void
.br
.BI "pthread_cleanup_pop(int " execute )
.SH DESCRIPTION
The
.B pthread_cleanup_push()
function establishes a cleanup handler to be executed when the thread exits
or is cancelled.  The
.B pthread_cleanup_pop()
function removes the cleanup handler at the top of the cleanup stack and,
if the parameter,
.B execute,
is nonzero, causes the handler to be executed.
.SH PARAMETERS
.SS Return Parameters
.IP None
.SS Input Parameters
.TP
.B \(bu
.B handler
is the address of a function to be executed when the calling thread terminates.
.TP
.B \(bu
.B arg
is an address of a location in memory containing an argument to the cleanup
handler.  When the cleanup handler is invoked, this address is passed to the
cleanup handler.  This value may be NULL.
.TP
.B \(bu
.B execute
is an integer valued parameter specifying whether the cleanup routine in
.B pthread_cleanup_push()
is to executed when the thread terminates normally or is cancelled.  If the
value of
.B execute
is 0, the routine is executed only if the thread is cancelled or otherwise
terminates abnormally.  If its value is non-zero, the routine is executed
regardless of how the thread terminated.
.SH RETURN VALUE
None.
.SH USAGE EXAMPLE
.PP
Push a handler on stack of the calling thread.
.PP
.nf
    static void
    cleanup_handler( void *arg )
    {
        /*
         *  Cleanup state, print exit message, etc.,
         */
    }

    int
    thread_proc()
    {
        pthread_cleanup_push( cleanup_handler, NULL );
                    .
                    .
                    .
        pthread_cleanup_pop( TRUE );
        pthread_exit( (void *) SUCCESS );
    }

.fi
.PP
In this example, a cleanup handler is pushed onto the stack of every thread
that executes the
.B thread_proc()
function.  Later, immediately before the thread calls
.B pthread_exit(),
the cleanup handler is popped and, because the
.B execute
parameter is TRUE, the handler is caused to execute.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_testcancel(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec.com,  mtp@big.aa.net>

