\# $Id: pthread_delay.man,v 1.1.1.1 1996/06/29 01:20:50 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_delay.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:49  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:25  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:06  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_delay_np \- Block a thread for a specified interval of time
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_delay_np(const struct timespec *" interval )
.SH DESCRIPTION
The
.B pthread_delay_np()
function causes the calling thread to unconditionally block for an interval of
time specified by the
.I interval
parameter.
.SH PARAMETERS
.SS Input Parameters
.IP interval
The address of a variable specifying an interval of time (seconds and
nanoseconds) to wait.  The structure,
.I struct timespec
may be found in
.I /usr/include/typedefs.h
(furnished with the PCthreads V1.0 Kit).  Its layout is
.sp
.nf
            struct timespec
            {
                long tv_sec;
                long tv_nsec;           /* nanoseconds */
            };
.fi
.sp
To cause a thread to wait for 30.5 seconds, the
.I interval
parameter must contain the values
.I abs_time->tv_sec = 30
and
.I abs_time->tv_nsec = 500000
.SS Return Parameters
.IP None
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by the
.I interval
parameter is invalid (e.g., too large, or negative).
.TP
.IP ENOMEM
The process's heap has become exhausted.
.SH USAGE EXAMPLE
.PP
The following code fragment illustrates how to delay a thread for
30.5 seconds.
.PP
.sp
.nf
            struct timespec ts = {30, 500000};

                    .
                    .
            status = pthread_delay_np( &ts );
            CHECK(status, "Thread delay failed");
                    .
                    .
.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B related_function(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
