\# $Id: pthread_join.man,v 1.1.1.1 1996/06/29 01:20:50 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_join.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:50  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:26  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:06  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_join \- Wait for thread termination
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.B pthread_join( pthread_t thread, void **status )
.SH DESCRIPTION
The
.B pthread_join()
function retrieves the exit status of the target thread (i.e., specified by the
.I thread
parameter).  On successful return from
.B pthread_join(),
the target thread is detached and, if the
.I status
parameter is non-NULL, the value passed to
.B pthread_exit()
by the target thread is returned in the location it references. If the target
thread has not completed, the thread calling
.B pthread_join()
is suspended until the target thread completes.  To this end, the
.B pthread_join()
function can be correctly viewed as a mechanism by which a thread may 
synchronize its execution with the completion of another.
.PP
Threads are created, by default, as detached, i.e., they can not be joined. 
To create a thread that is joinable, the
.B pthread_attr_setdetachstate()
must be called with a value of
.B PTHREAD_CREATE_JOINABLE.
For a more detailed explaination of termination semantics, see the manual
page for
.B pthread_exit().
.SH PARAMETERS
.SS Input Parameters
.IP thread
The identifier of the target thread.
.SS Return Parameters
.IP status
The address of a pointer to a memory location into which the thread library
will copy the exit status of the target thread.  If this value is NULL, the
thread library ignores this parameter.
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The
.I thread
parameter references a thread that is not joinable (i.e., has been detached).
.TP
.IP ESRCH
The
.I thread
parameter does not refer to an existing thread.
.TP
.IP ENOMEM
The process's heap has become exhausted.
.TP
.SH USAGE EXAMPLE
.PP
Join with a target thread
.PP
.nf
#include <stdlib.h>
#include <pthread.h>

void
proc( int val )
{
	int completion_status = EXIT_SUCCESS;

	if( do_something() < 0 )
		completion_status = EXIT_FAILURE;
    else
	{
		if( do_something_else() < 0 )
			completion_status = EXIT_FAILURE;
			.
			.
			.
	}

	pthread_exit( (void *) completion_status );
}

static pthread_t th;
static pthread_attr_t attr;

int
main()
{
	int param = 1, exit_status;

	(void) pthread_attr_init( &attr );
	(void) pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_JOINABLE );

	(void) pthread_create( &th, &attr, proc, &param );

	(void) pthread_join( th, (void **) &exit_status );
	if( exit_status != EXIT_SUCCESS )
		print_str("proc() failed");

	return( exit_status );
}
.fi
Note: As a matter of good programming practice, the
.B pthread_join()
or
.B pthread_detach()
functions should be called for every thread created with the
PHREAD_CREATE_JOINABLE detach state attribute.  Also, this example assumes 
that print_str() is reentrant function that prints the string to stdout.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_exit(3), pthread_create(3), pthread_detach(3), 
.B pthread_attr_setdetachstate(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>







