\# $Id: pthread_key_create.man,v 1.1.1.1 1996/06/29 01:20:50 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_key_create.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:51  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:26  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:07  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_key_create \- Create a thread-specific data key
.br
pthread_key_delete \- Delete a thread-specific data key.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_key_create(pthread_key_t *" key ","
.br
.BI "                   void (*" destructor ")(void *" arg ")" "")
.sp
.B int
.br
.BI "pthread_key_delete(pthread_key_t " key )
.SH DESCRIPTION
The 
.B pthread_key_create()
routine generates a unique thread-specific data key visible to all threads in
POSIX process.  The variable,
.I key
, is implemented as an opaque object and is used to set and retrieve a thread's
instance of the data. When called, this routine creates and returns a new key
value.  Keys must be initialized once, and only once.  See the
.B pthread_once()
manual page for details on how this can be accomplished.
.sp
Any single key may be used by different threads because
the values bound to the key are maintained on per thread basis and persist for
the life of the calling thread.  For example, suppose a programmer wanted to
create a new attributed for an application's threads called "color".  The
.B pthread_key_create()
call is invoked to create the attribute (i.e., the key).  Then the
.B pthread_key_setspecific()
routine is used on each thread to set the thread's "color".
.sp
When a thread terminates, all thread-specifc data is automatically destroyed
by calling a destructor function specified by the
.I destructor
parameter (a pointer to the destructor function).  However, the key is not
deleted, just the data for that thread.  To delete a key, the
.B pthread_key_delete()
function is used.
function...
.SH PARAMETERS
.SS Input Parameters
.IP key
When
.I key
is the parameter to the
.B pthread_key_create()
routine,
.I key
represents the address into which the threads library will store the
key identifer.  
.I destructor
A pointer to a function that will be called to destroy a thread-
.SS Return Parameters
.IP key
When
.I key
is the parameter to the
.B pthread_key_delete()
function,
.I key
is the thread-specific data identifier returned by a previous call to the
.B pthread_key_create()
service.
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by the key parameter was invalid, or the address
was NULL.
.TP
.IP EAGAIN
The system lacked the necessary resources to create another thread-specific
key.  The total number of keys per process is specified by the
.B PTHREAD_KEYS_MAX
macro (see
.I pthread.h.
.TP
.IP ENOMEM
The process's heap has become exhausted.
.SH USAGE EXAMPLE
.PP
Create a thread-specific key and set and retrieve the values on a per-thread
basis.
.PP
.nf
    /*
     * Initialize two data keys for thread names and addresses.  Note,
     * this routine is passed to the pthread_once_init() routine.
     */
    static pthread_once_t key_once = PTHREAD_ONCE_INIT;

    static void
    init_key( void )
    {
        int st;

        st = pthread_key_create( &name_key, destroy_data );
        CHECK(st, "pthread_key_create()");

        st = pthread_key_create( &address_key, NULL );
        CHECK(st, "pthread_key_create()");
    }
                .
                .
                .
    static void
    thread_proc( full_name_t *thread_data )
    {
                .
                .
                .
        /*
         *  --  Set the name and address of the thread.
         */
        st = pthread_setspecific( name_key, thread_data->name );
        CHECK(st, "pthread_setspecific()");

        st = pthread_setspecific( address_key, thread_data->address );
        CHECK(st, "pthread_setspecific()");
                .
                .
                .
    }

    int
    main()
    {
        st = pthread_once( &key_once, init_key );
        CHECK(st, "pthread_once()");
                .
                .
                .
        /*
         * Create a bunch of threads and initialize a data structure
         * we use to pass the name and address to the newly created thread.
         */
        for(i = 0; i < thread_count; i++ )
        {
            full_name = malloc( sizeof( full_name_t ));
            sprintf( full_name->first, "%s_%d", "bob", i );
            sprintf( full_name->last, "%s_%d", "smith", i );

            st = pthread_create( &th[i],
                            &attr_h,
                            (thread_proc_t) thread_proc,
                            (void *) full_name );
            CHECK(st, "pthread_create() failed");
        }
                .
                .
                .
        /*
         * Join each thread and retrieve the thread-specific data.
         */
        for(i = 0; i < thread_count; i++ )
        {
            st = pthread_join( th[i], (void **) &exit_val );
            CHECK(st, "pthread_join()");

            st = pthread_getspecific( name_key, (void **) &name );
            CHECK(st, "pthread_getspecific()");

            st = pthread_getspecific( address_key, (void **) &addr );
            CHECK(st, "pthread_getspecific()");

            if( exit_val != SUCCESS )
            fprintf( stderr, "Failed to join with th[%d]!\n", i);
        }
.fi
.PP
In this example, two keys are created in the init_key() function.  
Thread-specific data is then passed to each thread and the
.B pthread_setspecific()
function is used to set that thread's
.I name
and
.I address.
.sp
When each thread completes, the main thread retrieves the thread-specific
values using the
.B pthread_getspecific()
function.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_getspecific(3), pthread_setspecific(3), pthread_once(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
