\# $Id: pthread_kill.man,v 1.1.1.1 1996/06/29 01:20:51 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_kill.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:51  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:26  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:07  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_kill \- Send a signal to a thread
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.B pthread_kill( pthread_t thread, int sig )
.SH DESCRIPTION
The
.B pthread_kill()
function generates a signal to be delivered to a specified thread.  If the
.I thread
parameter is NULL or is the value returned from the
.B pthread_self(3)
function, the signal value
.I sig
is sent to the calling thread.  If
.I sig
= 0, only error checking is performed (NB:
.B pthread_kill
( thread, 0 ) can be used to test for the existence of a specified thread.
If the thread does not exist, the function returns EINVAL.
.PP
This function generates, but not necessarily delivers, the signal.
For example, if the target thread has the signal blocked, or is not active
(e.g., perhaps it is queued for a mutex) the signal is added to 
the thread's set of pending signals.  The signal will not be delivered
until the thread regains a processor.
.SH PARAMETERS
.SS Input Parameters
.IP thread
The identifier of the thread to which the signal is to be delivered. If this
parameter is NULL, the signal is generated for the calling thread.
.IP sig
The signal to be generated and delivered to the target thread.  If
.I sig
is zero, error checking is performed, but no signal is sent.  If
.I sig
represents a synchronous signal, the
.I thread
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by one or more of the parameters is invalid.  This usually
means that the signal value is invalid.  If
.I sig
is zero, however, this return value means the target thread no longer exists.
.SH USAGE EXAMPLE
.PP
Terminate the process by sending SIGKILL to the process.
.PP
.nf
#include <pthread.h>

int terminate_abrubtly = FALSE;

/*
 * This procedure is executed by dedicated thread whose sole function
 * is to wait until it is told (via pthread_cond_signal()) that it is
 * to terminate the process via SIGTERM.
 */
void
the_terminator( void )
{
	/*
	 * Wait on a condition variable until such time as it is necessary
	 * to terminate the process.
	 */
	(void) pthread_mutex_lock( &guard );
	while( !terminate_abrubtly )
		(void) pthread_cond_wait( &cv, &mu );

	(void) pthread_mutex_unlock( &mu );

	/*
	 * Asta la vista, baby...
	 */
	(void) pthread_kill( pthread_self(), SIGKILL );

	pthread_exit( (void *) SUCCESS ); /* Should not get here */
}
.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_sigmask(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
