\# $Id: pthread_lock_global_np.man,v 1.1.1.1 1996/06/29 01:20:51 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_lock_global_np.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:51  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:27  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:07  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_lock_global_np \- Acquire the global mutex
.br
pthread_unlock_global_np \- Release the global mutex.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.B pthread_lock_global_np( void )
.PP
.B int
.br
.B pthread_unlock_global_np( void )
.SH DESCRIPTION
These two services lock and unlock a global mutex shared among
.I all
software
components of an executable program.  This mutex is defined and initialized
by the thread library runtime.  Because it is global, it is available
to all multithreaded Linux applications that may have to link into libraries 
that are not thread-safe.  One of the most important semantics of the global 
mutex is its recursive nature. To this end, the global mutex can be relocked
by a thread holding the global mutex without deadlocking.  Thus, the recursive
nature of the mutex allows threads to call into other multithreaded libraries
without fear of deadlocking.  Of course, the locking thread must call
.B pthread_unlock_global_np()
as many times as it called
.B pthread_lock_global_np().
.PP
The need for a recursive global mutex should decline as more and more libraries
become thread-safe.  Hence, this service is not required by POSIX .1c.  As
a consequence the use of this service is not portable (hence the
.B _np
suffix).
.PP
Upon return from
.B pthread_lock_global_np(),
the global mutex is locked and held by the calling thread.  If the global
mutex is held by another thread, the calling thread will block until the global
mutex is unlocked by a call to
.B pthread_unlock_global_np().
At that time ownership of the mutex is transferred to the highest priority
thread blocked for that mutex.
.SH PARAMETERS
.SS Input Parameters
.IP None
.SS Return Parameters
.IP None
.SH RETURN VALUE
A value of zero (0) is always returned.
.SH USAGE EXAMPLE
.PP
Recursively protect calls into other other libraries
.PP
.nf
#include <pthread.h>

void
print_str( const char *str )	/* Function from Fred's library */
{
	pthread_global_lock_np();
	printf("%s\n", str );
	pthread_global_lock_np();
}

void							/* Function from Barbara's library */
bar( void )
{
	pthread_lock_global_np()
	print_str("What's up, doc?\n");
	pthread_unlock_global_np();
}

int
foo( void )						/* Function from Bob's library */
{
	pthread_lock_global_np();
	print_str("Gizbah?\n");
	pthread_unlock_global_np();
		.
		.
		.
}
.fi
.B NB:
In this example, note that the call to printf(), a non-reentrant function
on Linux and many other systems, is protected by the global lock.  Moreover,
because the mutex is recursive a thread may lock the same mutex as it
descends the call stack.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_mutex_lock(3), pthread_mutex_unlock(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>







