\# $Id: pthread_sigmask.man,v 1.1.1.1 1996/06/29 01:20:51 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_sigmask.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:53  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:28  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:09  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_sigmask \- Examine or modify the calling thread's signal mask.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.B pthread_sigmask( 
.br
.B				int how
.br
.B				sigset_t *new,
.br
.B				sigset_t *old )
.SH DESCRIPTION
The
.B pthread_sigmask()
function replaces sigprocmask() in multithreaded (POSIX .1c) applications.  In
Linux threads, all calls to sigprocmask() are replaced with
.B pthread_sigmask()
by the C-language preprocessor.
.PP
In POSIX .1c conforming applications signal semantics have changed
substantially.  POSIX .1c applications no longer have a process-wide
signal mask, hence the irrelevance of sigprocmask().  All masks are per-thread.
If sigprocmask() is used, it will return the signal mask of the calling thread,
not the process-wide signal mask.
.PP
.B NB:
While process-wide signal masks are no longer supported, process-wide signal 
handlers installed using
.B sigaction()
still are.  In otherwords, masks are per thread, but signal handlers are
shared among the threads of a process. Thus, if thread 1 sets the
.B sa_handler
field of the thread's SIGTERM sigaction to SIG_DFL and then thread 2 
[re]sets the sa_handler member to SIG_IGN, when thread 1 takes a SIGSEGV, 
the kernel will deliver the SIGSEGV according to the SIG_IGN semantics.
.SH PARAMETERS
.SS Input Parameters
.IP how
Specifies how the signal mask represented by the
.I new
parameter is to be combined with the calling thread's current signal mask. The
allowed values of
.I how
are:
.RS
.TP
.B \(bu
.I SIG_SETMASK
specifies that the value pointed to by
.I new
is to replace the calling thread's current signal set.
.TP
.B \(bu
.I SIG_BLOCK
specifies that the signals specified by the value pointed to by
.I new
is to be added to the calling thread's current signal set.
.TP
.B \(bu
.I SIG_UNBLOCK
specifies that the signals specified by the value pointed to by
.I new
are to be unblocked in the calling thread's current set.
.RE
.IP new
This parameter is the address of the signal mask to be used to modify or
examine the current thread's signal mask.
.SS Return Parameters
.IP old
This parameter is an address in memory into which the
.pthread_sigmask()
function will copy the calling thread's signal mask before modification.  The
.I old
parameter may be NULL.
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The values specified by
.I how
and
.I new
are invalid.
.TP
.IP ENOMEM
The process's heap has become exhausted.
.SH USAGE EXAMPLE
.PP
Block all signals except SIGSEGV
.PP
.nf
#include <pthread.h>

int
main()
{
	sigset_t set;

	sigfillset( &set );
	sigdelset( &set, SIGSEGV );
	pthread_sigmask( SIG_BLOCK, &set, NULL );
		.
		.
		.
}
.fi
.B NB:
Observe that because this mask, having been created in the main thread, will
be inherited by all subsequent threads.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B sigwait(3), sigfillset(3), sigemptyset(3), sigdelset(3), sigaddset(3)
.B sigprocmask(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>








