\# $Id: pthread_sigwait.man,v 1.1.1.1 1996/06/29 01:20:51 mtp Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_sigwait.man,v $
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:53  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:28  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:09  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
sigwait \- Wait for an asynchronous signal
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "sigwait(sigset_t " set )
.SH DESCRIPTION
The
.B sigwait()
function determines whether any of the signals specified in
.I set
are pending and atomically clears that signal from the system's set of pending 
signals and returns that signal number.  If no signal in
.I set
is pending at the time of the call, the calling thread is suspended until one
or more of the signals in
.I set
become pending.  The signals defined in
.I set
must be blocked at the time of the call to
.BI sigwait().
The effect of
.BI sigwait()
on the signal actions for the signals in
.set
is unspecified.
.PP
If more than one thread is using
.BI sigwait()
to wait for the same signal, only one of these threads will return.  Which
thread returns, if more than one thread is waiting is unspecified.
.SH PARAMETERS
.SS Input Parameters
.IP set
In Linux (and most other 32-bit UNIX systems), this is an integer valued
variable used as a bit mask.  For information about initializing the
.I set
parameter see either of the sigprocmask(3), sigemptyset(3), sigfillset(3),
sigaddset(3), or sigdelset(3) function references.
.SH RETURN VALUE
Upon successful completion,
.BI sigwait()
returns the signal number of the received signal.  Otherwise, -1 (FAILURE)
is returned and errno is set to one of the values below:
.IP ENOMEM
The process's heap has become exhausted.
.TP
.IP EINVAL
The
.set
parameter contains an invalid or unsupported signal number.
.RS 
.SH USAGE EXAMPLE
.PP
The thread procedure below,
.BI waiter(),
blocks the thread until one of the signals specified in the
.I set
parameter is delivered to the process.
.PP
.nf
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

void
waiter( void )
{
	sigset_t set;
	int caught;

	sigemptyset( &set );
	sigaddset( &set, SIGINT );
	sigaddset( &set, SIGTERM );
	sigaddset( &set, SIGHUP );
	sigaddset( &set, SIGQUIT );

    pthread_sigmask( SIG_BLOCK, &set, NULL );

	switch((caught = sigwait( sigset )))
	{
	  case SIGINT:
		printf("Caught SIGINT\n");
		break;
	  case SIGTERM:
		printf("Caught SIGTERM\n");
		break;
	  case SIGHUP:
		printf("Caught SIGHUP\n");
		break;
	  case SIGQUIT:
		printf("Caught SIGQUIT\n");
		break;
	  default:
		caught = FAILURE;
		print_str("Error");
		break;
	}

	pthread_exit( (void *) caught );
}
.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_sigmask(3), sigprocmask(3), sigemptyset(3),
.B sigfillset(3), sigaddset(3), sigdelset(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
