#ifndef lint
static const char rcsid[] = "$Id: cleanup.c,v 1.1.1.1 1996/06/29 01:20:51 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: cleanup.c,v $
 * Revision 1.1.1.1  1996/06/29 01:20:51  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:57:58  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:40  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:27  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include "dbgmem.h"
#include "cleanup.h"

void
push_cleanup( handler_stack_t *stack, cleanup_t func, void *arg )
{
   handler_node_t *node;
   node = MALLOC( sizeof( handler_node_t ));
   node->handler = func;

   node->arg = arg;

   node->next = stack->first;
   stack->first = node;
   stack->elements += 1;
}

int
pop_cleanup( handler_stack_t *stack, int must_execute  )
{
   handler_node_t *node;
   cleanup_t func;
   void *arg;

   if( stack->elements == 0 || stack->first == NULL )
   {
       stack->elements = 0;
       return( 0 );
   }
   
   node = stack->first;
   stack->first = node->next;

   func = node->handler;
   arg = node->arg;

   /*
	* We need to deallocate the node containing the pointer to the cleanup
	* handler before we execute the handler.  Otherwise, a memory leak could
	* occur if the cleanup handler longjmp'd out of the handler and execution
	* is resumed in "user" mode (This, of course, is precisely what happens 
	* during exception handling as the call stack is being unwound).
	*/
   FREE( node );
   stack->elements -= 1;
   if( must_execute )
   {
       int flag;

       flag = sys_set_critical();      /* Save critical flag */
       sys_restore_critical( 0 );      /* Allow execution in "user" mode */
	   (*func)(arg);
       sys_restore_critical( 0 );      /* Restore critical flag */
   }

   return( stack->elements );
}






