#ifndef lint
static const char rcsid[] = "$Id: delay.c,v 1.1.1.1 1996/06/29 01:20:52 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: delay.c,v $
 * Revision 1.1.1.1  1996/06/29 01:20:52  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:58:01  mtp
 * + Release 2.0
 *
 * Revision 1.4  1996/01/14 20:22:41  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.3  1996/01/14 18:32:29  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.2  1995/12/31 06:12:59  mtp
 * + Added casts to accomodate the change in ev->ptr data type (now points
 *   to NULL).
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include "delay.h"
#include "machdep.h"
#include "timer_q.h"
#include "error.h"
#include "nub.h"
#include "priority_q.h"

/*
 * --  This service causes the calling thread to enqueue on the runtime's
 *     thread timer queue for "delay_interval" microseconds.
 */
int
thread_delay( counter_t delay_interval )
{
   struct PTHREAD_HANDLE *active_th;
   timer_event_t *ev;

   ev = MALLOC( sizeof( timer_event_t ));
   if( ev == NULL )
   {
       set_thread_errno( ENOMEM );
       return( ENOMEM );
   }

   active_th = nub_get_active_thread();
   ev->next = NULL;
   ev->prev = NULL;
   ev->handle = active_th;
   ev->time = delay_interval + nub_get_elapsed_usecs();

   insert_timer_event( ev );
   active_th->obj.thread_state = THREAD_DELAYED_C;
   nub_reschedule( active_th );   
   return( SUCCESS );
}

/*
 * --  This service is only called by the clock interrupt routine.
 */
int
thread_unwait( counter_t current_time )
{
   int unwaited_threads = 0;
   timer_event_t *ev;
   struct PTHREAD_HANDLE *handle;

   while( check_expiration_time() <= (time_t)current_time )
   {
       ev = remove_timer_event();
       handle = (struct PTHREAD_HANDLE *) ev->handle;
       handle->obj.thread_state = THREAD_READY_C;
       enqueue_thread( &thread_rq, handle );
       FREE( ev );
       unwaited_threads += 1;
   }

   return( unwaited_threads );
}
