#ifndef lint
static const char rcsid[] = "$Id: error.c,v 1.1.1.1 1996/06/29 01:20:52 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: error.c,v $
 * Revision 1.1.1.1  1996/06/29 01:20:52  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:58:01  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:42  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:29  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include <errno.h>
#include "private.h"
#include "error.h"
#include "nub.h"

typedef struct ERROR_MSG_TABLE
{
   error_code_t    error;
   char            *error_msg;

} error_msg_table_t;

static error_msg_table_t table[] = 
{
   {ERR_C_MIN,              ""},
   {ERR_C_EXIT_W_LOCK,      "Thread exited while holding mutex"},
   {ERR_C_CANCEL_W_LOCK,    "Thread canceled while holding mutex"},
   {ERR_C_SIG_NOT_ALLOWED,  "async signal has no effect - use sigwait()"},
   {ERR_C_MAX,              "Unknown error code"}
};
   
char *
get_system_err_msg( error_code_t code )
{
   if( code <= ERR_C_MIN && code >= ERR_C_MAX )
       code = ERR_C_MAX;
       
   return( table[code].error_msg );
}

void
errlog( FILE *fd, const char *msg )
{
   if( fd == NULL )
       fd = stdout;

   fprintf( fd, "log: %s\n", msg );
}

void
set_thread_errno( int errno_value )
{
   struct PTHREAD_HANDLE *active_th;

   active_th = nub_get_active_thread();
   active_th->obj.errno = errno_value;
   errno = 0;
}

int
get_thread_errno( struct PTHREAD_HANDLE *th_h )
{
   return( th_h->obj.errno );
}
