#ifndef lint
static const char rcsid[] = "$Id: pthread_condattr.c,v 1.1.1.1 1996/06/29 01:20:53 mtp Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: pthread_condattr.c,v $
 * Revision 1.1.1.1  1996/06/29 01:20:53  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:58:14  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:55  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:45  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include "pthread.h"
#include "public.h"
#include "condv_attr.h"
#include "error.h"

extern condv_attr_obj_t *
get_default_cv_attr_obj( void );

/*------------------------------------------------------------------------*/
/*                CONDITION VARIABLE ATTR SERVICES                        */
/*------------------------------------------------------------------------*/
int  
pthread_condattr_init( pthread_condattr_t  *handle )
{
   int flag, st;
   struct PTHREAD_CONDATTR_HANDLE *attr_h;

   if( (st = system_init()) != SUCCESS )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   /*
    *  --  Create an attributes handle initialized with default values.
    */
   if( (attr_h = allocate_handle( CONDV_ATTR_C )) == NULL )
   {
       set_thread_errno( ENOMEM );
       RETURN( ENOMEM );
   }

   attr_h->type = CONDV_ATTR_C;

   if( (attr_h->obj = get_default_cv_attr_obj()) == NULL )
   {
       set_thread_errno( ENOMEM );
       RETURN( ENOMEM );
   }

   *handle = attr_h;
   RETURN( SUCCESS );
}

int  
pthread_condattr_destroy( pthread_condattr_t  *handle )
{
   int flag, st;

   if( (st = system_init()) != SUCCESS )
       return( st );

   /*
    *  --  Ensure that the handle is valid.
    */
   sys_disable_ctxsw( flag );
   if( handle == NULL || (*handle)->type != ATTR_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   destroy_cv_attr_obj( (*handle)->obj );
   free_handle( *handle );
   handle = NULL;
   
   RETURN( SUCCESS );
}
